package com.rf.kjb.chat.service.impl;

import com.rf.kjb.chat.dao.domain.ChatEntity;
import com.rf.kjb.chat.dao.domain.ChatRecordEntity;
import com.rf.kjb.chat.dao.repository.ChatRecordRepository;
import com.rf.kjb.chat.service.ChatRecordService;
import com.rf.kjb.scale.dao.model.CategoryEntry;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author:zzf
 * @Date:2023/9/14:15:27
 * @Description:
 */
@Service
public class ChatRecordServiceImpl implements ChatRecordService {
    @Autowired
    private ChatRecordRepository repository;
    @Override
    public void save(ChatRecordEntity chatRecordEntity) {
        repository.save(chatRecordEntity);
    }

    @Override
    public Page<ChatEntity> find(String identifier, String beginDate,String endDate, int pageNum, int pageSize) {
        Specification<ChatEntity> specification = (root, query, cb) -> {
            List<javax.persistence.criteria.Predicate> predicateList = new ArrayList<>();
            if(StringUtils.isNotBlank(identifier)){
                predicateList.add(cb.equal(root.get("identifier"),identifier));
            }

            if(StringUtils.isNotBlank(beginDate)){
                cb.greaterThanOrEqualTo(root.get("createTime"),beginDate);
            }

            if(StringUtils.isNotBlank(endDate)){
                cb.lessThanOrEqualTo(root.get("createTime"),endDate);
            }

            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.asc(root.get("createTime")));
            return query.getRestriction();
        };
        if (pageNum <= 0) {
            pageNum = 1;
        }

        if(pageSize <= 0){
            pageSize = 10;
        }
        return this.repository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }
}
